# Version
# (fr) Version

from lazpaint import command, dialog
import sys

lazpaint_version = command.get_version()
python_version = sys.version_info

dialog.show_message("Python " + str(python_version[0]) + "." + str(python_version[1]) + "." + str(python_version[2]) + ", " + "LazPaint " + str(lazpaint_version[0]) + "." + str(lazpaint_version[1]) + "." + str(lazpaint_version[2]) )




# Make thumbnails of JPG as PNG format
from lazpaint import imagelist, image, dialog, view

THUMBNAIL_WIDTH = 128
THUMBNAIL_HEIGHT = 128

path = dialog.show_directory_dialog("Select a directory of pictures")
imagelist.clear()
imagelist.set_auto_uncheck_mode(imagelist.UNCHECK_ON_SAVE)
imagelist.set_auto_zoom_fit(True)
imagelist.add_files(path + "/*.jpg")
if imagelist.get_file_count() == 0:
  dialog.show_message("No JPG file found")
  exit()

thumbnails = []
for filename in imagelist.iterate(True):
  width, height = image.get_size()
  ratio_x = THUMBNAIL_WIDTH / width
  ratio_y = THUMBNAIL_HEIGHT / height
  ratio = min(ratio_x, ratio_y)
  image.resample(max(1, width*ratio), max(1, height*ratio))
  view.set_zoom()
  new_filename = image.change_file_extension(filename, "png")
  image.export(file_name=new_filename, validate=True, overwrite=False, skip_options=True)
  thumbnails.append(new_filename)

imagelist.clear()
imagelist.add_files(thumbnails)
imagelist.open_first(True)




import collections, math
from lazpaint import dialog, command, filters

GAMMA = 2.2
ALPHA_OPAQUE = 255
ALPHA_TRANSPARENT = 0

if __name__ == "__main__":
  dialog.show_message("Library defining colors.")

def to_linear(std_value: int): #0..255
  return math.pow(std_value/255, GAMMA)

def to_std(linear_value: float):
  return round(math.pow(linear_value, 1/GAMMA)*255)

CustomRGBA = collections.namedtuple("RGBA", "red, green, blue, alpha")
class RGBA(CustomRGBA):
  def __repr__(self):
    if self.alpha != 255: 
      return '#{:02X}{:02X}{:02X}{:02X}'.format(self.red,self.green,self.blue,self.alpha) 
    else:
      return '#{:02X}{:02X}{:02X}'.format(self.red,self.green,self.blue) 
  def __str__(self):
    return '{:02X}{:02X}{:02X}{:02X}'.format(self.red,self.green,self.blue,self.alpha)
  def __eq__(self, other):
    if isinstance(other, RGBA):
      return (other.red == self.red) and (other.green == self.green) and (other.blue == self.blue) and (other.alpha == self.alpha)
    else:
      return NotImplemented
  def negative(self):
    return RGBA(to_std(1-to_linear(self.red)), to_std(1-to_linear(self.green)), to_std(1-to_linear(self.blue)), self.alpha)
  def linear_negative(self):
    return RGBA(255-self.red, 255-self.green, 255-self.blue, self.alpha)
  def swap_red_blue(self):
    return RGBA(self.blue, self.green, self.red, self.alpha)
  def grayscale(self):
    gray = to_std(to_linear(self.red)*0.299 + to_linear(self.green)*0.587 + to_linear(self.blue)*0.114)
    return RGBA(gray, gray, gray, self.alpha)

def RGB(red: int, green: int, blue: int): #0..255
  return RGBA(red, green, blue, 255)

def str_to_RGBA(s):
  if s is None:
    return None
  elif isinstance(s, list):
    return [str_to_RGBA(c) for c in s]
  else:
    if s[0:1] == "#":
      s = s[1:]
    if len(s) == 6:
      return RGBA(int(s[0:2],16), int(s[2:4],16), int(s[4:6],16), 255)
    elif len(s) == 8:
      return RGBA(int(s[0:2],16), int(s[2:4],16), int(s[4:6],16), int(s[6:8],16))
    else:
      raise ValueError("Invalid color string")

TRANSPARENT = RGBA(0,0,0,0)

#VGA color names
BLACK = RGB(0,0,0)
BLUE = RGB(0,0,255)
LIME = RGB(0,255,0)
AQUA = RGB(0,255,255)
RED = RGB(255,0,0)
FUCHSIA = RGB(255,0,255)
ORANGE = RGB(255,165,0)
YELLOW = RGB(255,255,0)
WHITE = RGB(255,255,255)
GRAY = RGB(128,128,128)
NAVY = RGB(0,0,128)
GREEN = RGB(0,128,0)
TEAL = RGB(0,128,128)
MAROON = RGB(128,0,0)
PURPLE = RGB(128,0,128)
OLIVE = RGB(128,128,0)
SILVER = RGB(192,192,192)

def get_curve(points: list, posterize=False):
  return {'Points': points, 'Posterize': posterize}

def curves(red=[], red_posterize=False, green=[], green_posterize=False, blue=[], blue_posterize=False, hue=[], hue_posterize=False, saturation=[], saturation_posterize=False, lightness=[], lightness_posterize=False, validate=True):
  command.send('ColorCurves', Red=get_curve(red, red_posterize), Green=get_curve(green, green_posterize), Blue=get_curve(blue, blue_posterize), Hue=get_curve(hue, hue_posterize), Saturation=get_curve(saturation, saturation_posterize), Lightness=get_curve(lightness, lightness_posterize), Validate=validate)

def posterize(levels=None, by_lightness=True, validate=True):
  command.send('ColorPosterize', Levels=levels, ByLightness=by_lightness, Validate=validate)

def colorize(hue_angle=None, saturation=None, correction=None, validate=True): #saturation: 0..1
  command.send('ColorColorize', Hue=hue_angle, Saturation=saturation, Correction=correction, Validate=validate)

def complementary():
  filters.run(filters.COLOR_COMPLEMENTARY)

def shift(hue_angle=None, saturation=None, correction=None, validate=True): #saturation shift: -2..2
  command.send('ColorShiftColors', Hue=hue_angle, Saturation=saturation, Correction=correction, Validate=validate)

def intensity(factor=None, shift=None, validate=True): #factor and shift: -2..2
  command.send('ColorIntensity', Factor=factor, Shift=shift, Validate=validate)

def lightness(factor=None, shift=None, validate=True): #factor and shift: -2..2
  command.send('ColorLightness', Factor=factor, Shift=shift, Validate=validate)

def negative():
  filters.run(filters.COLOR_NEGATIVE)

def linear_negative():
  filters.run(filters.COLOR_LINEAR_NEGATIVE)

def normalize():
  filters.run(filters.COLOR_NORMALIZE)

def grayscale():
  filters.run(filters.COLOR_GRAYSCALE)

def show_dialog(color=None): #-> RGBA
  result_str = dialog.show_color_dialog(color)
  if result_str is None or len(result_str) == 0:
    return None
  else: 
    return str_to_RGBA(result_str)






# Split RGB channels (using Pillow)
from lazpaint import image, dialog, layer

try:
    from PIL import Image
except ImportError:
    dialog.show_message("Please install Pillow for Python.")
    exit()

# check if it is a channel
if layer.get_registry("split-channel") is not None:
  dialog.show_message("Layer already split")
  exit()

layer_id = layer.get_id()
layer_index = image.get_layer_index()
layer_opacity = layer.get_opacity()
layer_transparent = layer.is_transparent()

# check if it has been split
if layer.get_registry("split-channels-id") is not None:
  for i in range(1, layer.get_count()):
    image.select_layer_index(i)
    if layer.get_registry("split-source-id") == layer_id:
      dialog.show_message("Layer already split")
      exit()
  layer.select_id(layer_id)

temp_name = image.get_temporary_name()
temp_name = layer.save_as(temp_name)
if temp_name is None:
  dialog.show_message("Failed to retrieve layer")
  exit()

im = Image.open(temp_name)
width, height = im.size

zero = Image.new("L", (width, height))
r = Image.merge("RGB", [im.getchannel("R"), zero, zero])
g = Image.merge("RGB", [zero, im.getchannel("G"), zero])
b = Image.merge("RGB", [zero, zero, im.getchannel("B")])

image.do_begin()

layer.select_id(layer_id)
layer.set_visible(False)

channels = [(r, "Red", "R"), (g, "Green", "G"), (b, "Blue", "B")]
if layer_transparent:
  a = im.getchannel("A")
  channels.insert(0, (a, "Alpha", "A"))

channels_id = []
for ch in channels:
  ch[0].save(temp_name, "BMP")
  layer.add_from_file(temp_name)
  layer.set_name(ch[1] + " channel")
  layer.set_opacity(layer_opacity)
  if ch[2] == "A":
    layer.set_blend_op(layer.BLEND_MASK)
  elif ch[2] != channels[-1][2]:
    layer.set_blend_op(layer.BLEND_LIGHTEN)
  layer.set_registry("split-channel", ch[2])
  layer.set_registry("split-source-id", layer_id)
  channels_id.append(layer.get_id())
  image.move_layer_index(image.get_layer_index(), layer_index+1)

layer.select_id(layer_id)
layer.set_registry("split-channels-id", channels_id)
image.do_end()



from lazpaint import command, colors

HAND = 'Hand'
HOT_SPOT = 'HotSpot'
MOVE_LAYER = 'MoveLayer'
ROTATE_LAYER = 'RotateLayer'
ZOOM_LAYER = 'ZoomLayer'
PEN = 'Pen'
BRUSH = 'Brush'
CLONE = 'Clone'
COLOR_PICKER = 'ColorPicker'
ERASER = 'Eraser'
EDIT_SHAPE = 'EditShape'
RECTANGLE = 'Rect'
ELLIPSE = 'Ellipse'
POLYGON = 'Polygon'
SPLINE = 'Spline'
FLOOD_FILL = 'FloodFill'
GRADIENT = 'Gradient'
PHONG_SHAPE = 'Phong'
SELECT_PEN = 'SelectPen'
SELECT_RECT = 'SelectRect'
SELECT_ELLIPSE = 'SelectEllipse'
SELECT_POLY = 'SelectPoly'
SELECT_SPLINE = 'SelectSpline'
MOVE_SELECTION = 'MoveSelection'
ROTATE_SELECTION = 'RotateSelection'
MAGIC_WAND = 'MagicWand'
DEFORMATION_GRID = 'Deformation'
TEXTURE_MAPPING = 'TextureMapping'
LAYER_MAPPING = 'LayerMapping'
TEXT = 'Text'

#click state
STATE_LEFT = 'Left'
STATE_RIGHT = 'Right'
STATE_SHIFT = 'Shift'
STATE_ALT = 'Alt'
STATE_CTRL = 'Ctrl'

ERASER_MODE_ALPHA = 'EraseAlpha'
ERASER_MODE_SHARPEN = 'Sharpen'
ERASER_MODE_SOFTEN = 'Soften'
ERASER_MODE_LIGHTEN = 'Lighten'
ERASER_MODE_DARKEN = 'Darken'

PEN_STYLE_SOLD = 'Solid'
PEN_STYLE_DASH = 'Dash'
PEN_STYLE_DOT = 'Dot'
PEN_STYLE_DASH_DOT = 'DashDot'
PEN_STYLE_DASH_DOT_DOT = 'DashDotDot'

JOIN_STYLE_BEVEL = 'Bevel'
JOIN_STYLE_MITER = 'Miter'
JOIN_STYLE_ROUND = 'Round'

LINE_CAP_ROUND = 'Round'
LINE_CAP_SQUARE = 'Square'
LINE_CAP_FLAT = 'Flat'

FONT_STYLE_BOLD = 'Bold'
FONT_STYLE_ITALIC = 'Italic'
FONT_STYLE_UNDERLINE = 'Underline'
FONT_STYLE_STRIKE_OUT = 'StrikeOut'

ALIGN_LEFT = 'Left'
ALIGN_CENTER = 'Center'
ALIGN_RIGHT = 'Right'

BIDI_AUTO = 'BidiAuto'
BIDI_LEFT_TO_RIGHT = 'LeftToRight'
BIDI_RIGHT_TO_LEFT = 'RightToLeft'

SHAPE_OPTION_DRAW_SHAPE = 'DrawShape'
SHAPE_OPTION_FILL_SHAPE = 'FillShape'
SHAPE_OPTION_CLOSE_SHAPE = 'CloseShape'

ARROW_NONE = 'None'
ARROW_TAIL = 'Tail'
ARROW_TIP = 'Tip'
ARROW_NORMAL = 'Normal'
ARROW_CUT = 'Cut'
ARROW_FLIPPED = 'Flipped'
ARROW_FLIPPED_CUT = 'FlippedCut'
ARROW_TRIANGLE = 'Triangle'
ARROW_TRIANGLE_BACK1 = 'TriangleBack1'
ARROW_TRIANGLE_BACK2 = 'TriangleBack2'
ARROW_HOLLOW_TRIANGLE = 'HollowTriangle'
ARROW_HOLLOW_TRIANGLE_BACK1 = 'HollowTriangleBack1'
ARROW_HOLLOW_TRIANGLE_BACK2 = 'HollowTriangleBack2'

SPLINE_STYLE_INSIDE = 'Inside'
SPLINE_STYLE_INSIDE_WITH_ENDS = 'InsideWithEnds'
SPLINE_STYLE_CROSSING = 'Crossing'
SPLINE_STYLE_CROSSING_WITH_ENDS = 'CrossingWithEnds'
SPLINE_STYLE_OUTSIDE = 'Outside'
SPLINE_STYLE_ROUND_OUTSIDE = 'RoundOutside'
SPLINE_STYLE_VERTEX_TO_SIDE = 'VertexToSide'
SPLINE_STYLE_EASY_BEZIER = 'EasyBezier'

GRADIENT_TYPE_LINEAR = 'Linear'
GRADIENT_TYPE_REFLECTED = 'Reflected'
GRADIENT_TYPE_DIAMOND = 'Diamond'
GRADIENT_TYPE_RADIAL =  'Radial'
GRADIENT_TYPE_ANGULAR = 'Angular'

GRADIENT_INTERPOLATION_STD_RGB = 'StdRGB'
GRADIENT_INTERPOLATION_LINEAR_RGB = 'LinearRGB'
GRADIENT_INTERPOLATION_LINEAR_HSL_POSITIVE = 'LinearHSLPositive'
GRADIENT_INTERPOLATION_LINEAR_HSL_NEGATIVE = 'LinearHSLNegative'
GRADIENT_INTERPOLATION_CORR_HSL_POSITIVE = 'GSBPositive'
GRADIENT_INTERPOLATION_CORR_HSL_NEGATIVE = 'GSBNegative'

GRADIENT_REPETITION_PAD = 'Pad'
GRADIENT_REPETITION_REPEAT = 'Repeat'
GRADIENT_REPETITION_REFLECT = 'Reflect'
GRADIENT_REPETITION_SINE = 'Sine'

TEXTURE_REPETITION_NONE = 'None';
TEXTURE_REPETITION_X = 'RepeatX';
TEXTURE_REPETITION_Y = 'RepeatY';
TEXTURE_REPETITION_BOTH = 'RepeatBoth';

SHAPE_KIND_RECTANGLE = 'Rectangle'
SHAPE_KIND_ROUND_RECTANGLE = 'RoundRectangle'
SHAPE_KIND_HALF_SPHERE = 'HalfSphere'
SHAPE_KIND_CONE_TOP = 'ConeTop'
SHAPE_KIND_CONE_SIDE = 'ConeSide'
SHAPE_KIND_HORIZ_CYLINDER = 'HorizCylinder'
SHAPE_KIND_VERT_CYLINDER = 'VertCylinder'

DEFORMATION_MODE_DEFORM = 'Deform'
DEFORMATION_MODE_MOVE_POINT_WITHOUT_DEFORMATION = 'MovePointWithoutDeformation'

FLOOD_FILL_PROGRESSIVE = 'Progressive'
FLOOD_FILL_FILL_ALL = 'FillAll' #not implemented

PERSPECTIVE_REPEAT = 'Repeat'
PERSPECTIVE_TWO_PLANES = 'TwoPlanes'

KEY_UNKNOWN = 'Unknown'
KEY_BACKSPACE = 'Backspace'
KEY_TAB = 'Tab'
KEY_RETURN = 'Return'
KEY_ESCAPE = 'Escape'
KEY_PAGE_UP = 'PageUp'
KEY_PAGE_DOWN = 'PageDown'
KEY_HOME = 'Home'
KEY_END = 'End'
KEY_LEFT = 'Left'
KEY_UP = 'Up'
KEY_RIGHT = 'Right'
KEY_DOWN = 'Down'
KEY_INSERT = 'Insert'
KEY_DELETE = 'Delete'
KEY_NUM0 = 'Num0'
KEY_NUM1 = 'Num1'
KEY_NUM2 = 'Num2'
KEY_NUM3 = 'Num3'
KEY_NUM4 = 'Num4'
KEY_NUM5 = 'Num5'
KEY_NUM6 = 'Num6'
KEY_NUM7 = 'Num7'
KEY_NUM8 = 'Num8'
KEY_NUM9 = 'Num9'
KEY_F1 = 'F1'
KEY_F2 = 'F2'
KEY_F3 = 'F3'
KEY_F4 = 'F4'
KEY_F5 = 'F5'
KEY_F6 = 'F6'
KEY_F7 = 'F7'
KEY_F8 = 'F8'
KEY_F9 = 'F9'
KEY_F10 = 'F10'
KEY_F11 = 'F11'
KEY_F12 = 'F12'
KEY_A = 'A'
KEY_B = 'B'
KEY_C = 'C'
KEY_D = 'D'
KEY_E = 'E'
KEY_F = 'F'
KEY_G = 'G'
KEY_H = 'H'
KEY_I = 'I'
KEY_J = 'J'
KEY_K = 'K'
KEY_L = 'L'
KEY_M = 'M'
KEY_M = 'N'
KEY_O = 'O'
KEY_P = 'P'
KEY_Q = 'Q'
KEY_R = 'R'
KEY_S = 'S'
KEY_T = 'T'
KEY_U = 'U'
KEY_V = 'V'
KEY_W = 'W'
KEY_X = 'X'
KEY_Y = 'Y'
KEY_W = 'Z'
KEY_0 = '0'
KEY_1 = '1'
KEY_2 = '2'
KEY_3 = '3'
KEY_4 = '4'
KEY_5 = '5'
KEY_6 = '6'
KEY_7 = '7'
KEY_8 = '8'
KEY_9 = '9'
KEY_SHIFT = 'Shift'
KEY_CTRL = 'Ctrl'
KEY_ALT = 'Alt'

def choose(name):
  command.send("ChooseTool", Name=name)

def mouse(coords, state=[STATE_LEFT], default_pressure=1.0):
  if not isinstance(coords, list):
      xy = [(float(coords[0]), float(coords[1]))]
      if len(coords)>2:
        pressure = [float(coords[2])]
      else:
        pressure = [default_pressure]
  else:
      xy = [(float(c[0]), float(c[1])) for c in coords]
      pressure = [float(c[2]) if len(c)>2 else default_pressure for c in coords]      
  command.send("ToolMouse", Coords=xy, State=state, Pressure=pressure)

def keys(keys, state=[]):
  if isinstance(keys, str):
    keys = [keys]
  command.send("ToolKeys", Keys=keys, State=state)

def write(text):
  command.send("ToolWrite", Text=text)

def set_fore_color(color):
  command.send("ToolSetForeColor", Color=color)

def set_back_color(color):
  command.send("ToolSetBackColor", Color=color)

def set_outline_color(color):
  command.send("ToolSetOutlineColor", Color=color)

def get_fore_color():
  return colors.str_to_RGBA(command.send("ToolGetForeColor?"))

def get_back_color():
  return colors.str_to_RGBA(command.send("ToolGetBackColor?"))

def get_outline_color():
  return colors.str_to_RGBA(command.send("ToolGetOutlineColor?"))

def set_eraser_mode(mode):
  command.send('ToolSetEraserMode', Mode=mode)

def get_eraser_mode():
  return command.send('ToolGetEraserMode?')

def set_eraser_alpha(alpha: int): #0..255
  command.send('ToolSetEraserAlpha', Alpha=alpha)

def get_eraser_alpha() -> int:
  return command.send('ToolGetEraserAlpha?')

def set_pen_width(width: float):
  command.send('ToolSetPenWidth', Width=width)

def get_pen_width():
  return command.send('ToolGetPenWidth?')

def set_pen_style(style):
  command.send('ToolSetPenStyle', Style=style)

def get_pen_style():
  return command.send('ToolGetPenStyle?')

def set_join_style(style):
  command.send('ToolSetJoinStyle', Style=style)

def get_join_style():
  return command.send('ToolGetJoinStyle?')

def set_line_cap(cap):
  command.send('ToolSetLineCap', Cap=cap)

def get_line_cap():
  return command.send('ToolGetLineCap?')

def set_shape_options(options: list):
  command.send('ToolSetShapeOptions', Options=options)

def get_shape_options() -> list:
  return command.send('ToolGetShapeOptions?')

def set_aliasing(enabled: bool):
  command.send('ToolSetAliasing', Enabled=enabled)

def get_aliasing() -> bool:
  return command.send('ToolGetAliasing?')

def set_shape_ratio(ratio: float):
  command.send('ToolSetShapeRatio', Ratio=ratio)

def get_shape_ratio() -> float:
  return command.send('ToolGetShapeRatio?')

def set_brush_index(index: int):
  command.send('ToolSetBrushIndex', Index=index)

def get_brush_index() -> int:
  return command.send('ToolGetBrushIndex?')

def get_brush_count() -> int: 
  return command.send('ToolGetBrushCount?')

def set_brush_spacing(spacing: int):
  command.send('ToolSetBrushSpacing', Spacing=spacing)

def get_brush_spacing() -> int:
  return command.send('ToolGetBrushSpacing?')

def set_font_name(name: str):
  command.send('ToolSetFontName', Name=name)

def get_font_name() -> str:
  return command.send('ToolGetFontName?')

def set_font_size(size: float):
  command.send('ToolSetFontSize', Size=size)

def get_font_size() -> float:
  return command.send('ToolGetFontSize?')

def set_font_style(style: list):
  command.send('ToolSetFontStyle', Style=style)

def get_font_style() -> list:
  return command.send('ToolGetFontStyle?')

def set_text_align(align):
  command.send('ToolSetTextAlign', Align=align)

def get_text_align():
  return command.send('ToolGetTextAlign?')

def set_text_bidi_mode(bidi_mode):
  command.send('ToolSetTextBidiMode', BidiMode=bidi_mode)

def get_text_bidi_mode():
  return command.send('ToolGetTextBidiMode?')

def set_text_outline(width: float):
  command.send('ToolSetTextOutline', Width=width)

def get_text_outline() -> float:
  return command.send('ToolGetTextOutline?')

def set_text_phong(enabled: bool):
  command.send('ToolSetTextPhong', Enabled=enabled)

def get_text_phong() -> bool:
  return command.send('ToolGetTextPhong?')

def set_light_position(x: float, y: float):
  command.send('ToolSetLightPosition', Position=(x, y))

def get_light_position() -> tuple:
  return command.send('ToolGetLightPosition?')

def set_arrow_start(arrow):
  command.send('ToolSetArrowStart', Arrow=arrow)

def get_arrow_start():
  return command.send('ToolGetArrowStart?')

def set_arrow_end(arrow):
  command.send('ToolSetArrowEnd', Arrow=arrow)

def get_arrow_end():
  return command.send('ToolGetArrowEnd?')

def set_arrow_size(x, y):
  command.send('ToolSetArrowSize', Size=(x, y))

def get_arrow_size() -> tuple:
  return command.send('ToolGetArrowSize?')

def set_spline_style(style):
  command.send('ToolSetSplineStyle', Style=style)

def get_spline_style():
  return command.send('ToolGetSplineStyle?')

def set_fore_gradient_type(gradient_type):
  command.send('ToolSetForeGradientType', GradientType=gradient_type)

def get_fore_gradient_type():
  return command.send('ToolGetForeGradientType?')

def set_fore_gradient_colors(colors: list):
  command.send('ToolSetForeGradientColors', Colors=colors)

def get_fore_gradient_colors() -> list:
  return colors.str_to_RGBA(command.send('ToolGetForeGradientColors?'))

def set_fore_gradient_interpolation(interpolation):
  command.send('ToolSetForeGradientInterpolation', Interpolation=interpolation)

def get_fore_gradient_interpolation():
  return command.send('ToolGetForeGradientInterpolation?')

def set_fore_gradient_repetition(repetition):
  command.send('ToolSetForeGradientRepetition', Repetition=repetition)

def get_fore_gradient_repetition():
  return command.send('ToolGetForeGradientRepetition?')

def set_back_gradient_type(gradient_type):
  command.send('ToolSetBackGradientType', GradientType=gradient_type)

def get_back_gradient_type():
  return command.send('ToolGetGradientType?')

def set_back_gradient_interpolation(interpolation):
  command.send('ToolSetBackGradientInterpolation', Interpolation=interpolation)

def get_back_gradient_interpolation():
  return command.send('ToolGetBackGradientInterpolation?')

def set_back_gradient_colors(colors: list):
  command.send('ToolSetBackGradientColors', Colors=colors)

def get_back_gradient_colors() -> list:
  return colors.str_to_RGBA(command.send('ToolGetBackGradientColors?'))

def set_back_gradient_repetition(repetition):
  command.send('ToolSetBackGradientRepetition', Repetition=repetition)

def get_back_gradient_repetition():
  return command.send('ToolGetBackGradientRepetition?')

def set_outline_gradient_type(gradient_type):
  command.send('ToolSetOutlineGradientType', GradientType=gradient_type)

def get_outline_gradient_type():
  return command.send('ToolGetOutlineGradientType?')

def set_outline_gradient_colors(colors: list):
  command.send('ToolSetOutlineGradientColors', Colors=colors)

def get_outline_gradient_colors() -> list:
  return colors.str_to_RGBA(command.send('ToolGetOutlineGradientColors?'))

def set_outline_gradient_interpolation(interpolation):
  command.send('ToolSetOutlineGradientInterpolation', Interpolation=interpolation)

def get_outline_gradient_interpolation():
  return command.send('ToolGetOutlineGradientInterpolation?')

def set_outline_gradient_repetition(repetition):
  command.send('ToolSetOutlineGradientRepetition', Repetition=repetition)

def get_outline_gradient_repetition():
  return command.send('ToolGetOutlineGradientRepetition?')

def set_fore_texture(file_name):
  command.send('ToolSetForeTexture', FileName=file_name)

def set_fore_texture_repetition(repetition):
  command.send('ToolSetForeTextureRepetition', Repetition=repetition)

def get_fore_texture_repetition():
  return command.send('ToolGetForeTextureRepetition?')

def set_fore_texture_opacity(opacity: int):
  command.send('ToolSetForeTextureOpacity', Opacity=opacity)

def get_fore_texture_opacity(): #-> int 0..255
  return command.send('ToolGetForeTextureOpacity?')

def set_back_texture(file_name):
  command.send('ToolSetBackTexture', FileName=file_name)

def set_back_texture_repetition(repetition):
  command.send('ToolSetBackTextureRepetition', Repetition=repetition)

def get_back_texture_repetition():
  return command.send('ToolGetBackTextureRepetition?')

def set_back_texture_opacity(opacity: int):
  command.send('ToolSetBackTextureOpacity', Opacity=opacity)

def get_back_texture_opacity(): #-> int 0..255
  return command.send('ToolGetBackTextureOpacity?')

def set_outline_texture(file_name):
  command.send('ToolSetOutlineTexture', FileName=file_name)

def set_outline_texture_repetition(repetition):
  command.send('ToolSetOutlineTextureRepetition', Repetition=repetition)

def get_outline_texture_repetition():
  return command.send('ToolGetOutlineTextureRepetition?')

def set_outline_texture_opacity(opacity: int):
  command.send('ToolSetOutlineTextureOpacity', Opacity=opacity)

def get_outline_texture_opacity(): #-> int 0..255
  return command.send('ToolGetOutlineTextureOpacity?')

def set_phong_shape_kind(kind):
  command.send('ToolSetPhongShapeKind', Kind=kind)

def get_phong_shape_kind():
  return command.send('ToolGetPhongShapeKind?')

def set_grid_deformation_mode(mode):
  command.send('ToolSetDeformationGridMode', Mode=mode)

def get_grid_deformation_mode():
  return command.send('ToolGetDeformationGridMode?')

def set_flood_fill_options(options: list):
  command.send('ToolSetFloodFillOptions', Options=options)

def get_flood_fill_options() -> list:
  return command.send('ToolGetFloodFillOptions?')

def set_perspective_options(options: list):
  command.send('ToolSetPerspectiveOptions', Options=options)

def get_perspective_options() -> list:
  return command.send('ToolGetPerspectiveOptions?')




# Channels > Split RGB
# (fr) Canaux > Séparer RVB
from lazpaint import image, dialog, layer, filters

translation = dialog.translate_dict(["Layer already split", "Alpha", "Red", "Green", "Blue"])

# check if it is a channel
if layer.get_registry("split-channel") is not None:
  dialog.show_message(translation["Layer already split"])
  exit()

layer_id = layer.get_id()
layer_index = image.get_layer_index()
layer_opacity = layer.get_opacity()
layer_transparent = layer.is_transparent()

# check if it has been split
if layer.get_registry("split-channels-id") is not None:
  for cur_layer_id in image.iterate_layers():
    if layer.get_registry("split-source-id") == layer_id:
      dialog.show_message(translation["Layer already split"])
      exit()

image.do_begin()
channels = []
if layer_transparent:
  channels.append({"name": "Alpha", "channel": "A", "red": "alpha", "green": "alpha", "blue": "alpha", "alpha": "255"})
channels.append({"name": "Red", "channel": "R", "red": "red", "green": "0", "blue": "0", "alpha": "255"})
channels.append({"name": "Green", "channel": "G", "red": "0", "green": "green", "blue": "0", "alpha": "255"})
channels.append({"name": "Blue", "channel": "B", "red": "0", "green": "0", "blue": "blue", "alpha": "255"})

channels_id = [] 
for ch in channels:
  layer.select_id(layer_id)
  layer.duplicate()
  filters.filter_function(red = ch["red"], green = ch["green"], blue = ch["blue"], alpha = ch["alpha"], gamma_correction = False)
  layer.set_name(translation[ch["name"]])
  layer.set_opacity(layer_opacity)
  if ch["channel"] == "A":
    layer.set_blend_op(layer.BLEND_MASK)
  elif ch != channels[-1]:
    layer.set_blend_op(layer.BLEND_LIGHTEN)
  layer.set_registry("split-channel", ch["channel"])
  layer.set_registry("split-source-id", layer_id)
  channels_id.append(layer.get_id())

layer.select_id(layer_id)
layer.set_registry("split-channels-id", channels_id)
layer.set_visible(False)
image.do_end()



from lazpaint import tools, image, layer, dialog
import math, random

line_buf = []

def line(x, y, x2, y2):
  global line_buf
  if len(line_buf) > 0 and line_buf[-1] == (x, y):
    line_buf.append( (x2, y2) )
  else:
    flush_line()
    line_buf = [(x, y), (x2, y2)]

def flush_line():
  global line_buf
  if len(line_buf) > 0:
    tools.choose(tools.PEN)
    tools.mouse(line_buf)
  line_buf = []

MULTIPLIER = dialog.input_value("Zoom", 10)
DEG_TO_RAD = math.pi / 180

def drawTree(x1, y1, angle, depth):
    if (depth > 0):
        x2 = x1 + (math.cos(angle * DEG_TO_RAD) * depth * MULTIPLIER)
        y2 = y1 + (math.sin(angle * DEG_TO_RAD) * depth * MULTIPLIER)
        line(x1, y1, x2, y2)
        drawTree(x2, y2, angle - random.randint(15,50), depth - 1.44)
        drawTree(x2, y2, angle + random.randint(10,25), depth - 0.72)
        drawTree(x2, y2, angle - random.randint(10,25), depth - 3)
        drawTree(x2, y2, angle + random.randint(15,50), depth - 4)

image.do_begin()
layer.new()
drawTree(image.get_width() / 2, image.get_height(), -91, 7)
flush_line()
image.do_end()



# Channels > Split HSL
# (fr) Canaux > Séparer TSL
from lazpaint import image, dialog, layer, filters

translation = dialog.translate_dict(["Layer already split", "Hue", "Saturation", "Lightness", "Alpha": "Alpha"])

# check if it is a channel
if layer.get_registry("split-channel") is not None:
  dialog.show_message(translation["Layer already split"])
  exit()

layer_id = layer.get_id()
layer_index = image.get_layer_index()
layer_opacity = layer.get_opacity()
layer_transparent = layer.is_transparent()

# check if it has been split
if layer.get_registry("split-channels-id") is not None:
  for cur_layer_id in image.iterate_layers():
    if layer.get_registry("split-source-id") == layer_id:
      dialog.show_message(translation["Layer already split"])
      exit()

image.do_begin()
channels = []
if layer_transparent:
  channels.append({"name": "Alpha", "channel": "A", "hue": "0", "saturation": "0", "lightness": "alpha", "alpha": "255", "blend": layer.BLEND_MASK})
channels.append({"name": "Saturation", "channel": "S", "hue": "0", "saturation": "0", "lightness": "saturation", "alpha": "255", "blend": layer.BLEND_LINEAR_MULTIPLY_SATURATION})
channels.append({"name": "Lightness", "channel": "L", "hue": "0", "saturation": "0", "lightness": "lightness", "alpha": "255", "blend": layer.BLEND_HARD_LIGHT})
channels.append({"name": "Hue", "channel": "H", "hue": "hue", "saturation": "1", "lightness": "0.5", "alpha": "255", "blend": layer.BLEND_DRAW})

channels_id = [] 
for ch in channels:
  layer.select_id(layer_id)
  layer.duplicate()
  filters.filter_function(hue = ch["hue"], saturation = ch["saturation"], lightness = ch["lightness"], alpha = ch["alpha"], gamma_correction = False, corrected_hue = False)
  layer.set_name(translation[ch["name"]])
  layer.set_opacity(layer_opacity)
  if ch["channel"] != channels[-1]:
    layer.set_blend_op(ch["blend"])
  layer.set_registry("split-channel", ch["channel"])
  layer.set_registry("split-source-id", layer_id)
  channels_id.append(layer.get_id())

layer.select_id(layer_id)
layer.set_registry("split-channels-id", channels_id)
layer.set_visible(False)
image.do_end()



from lazpaint import command, dialog
import glob

if __name__ == "__main__":
  dialog.show_message("Library to act on the image list.")

UNCHECK_OFF = 'UncheckOff'
UNCHECK_ON_OPEN = 'UncheckOnOpen'
UNCHECK_ON_SAVE = 'UncheckOnSave'

def get_file_count() -> int:
  return command.send("ImageListGetFileCount?")

def get_selected_index() -> int:
  return command.send("ImageListGetSelectedIndex?")

def set_selected_index(index: int):
  command.send("ImageListSetSelectedIndex", Index=index)

def add_files(file_names: list) -> int:
  if isinstance(file_names, str):
    file_names = glob.glob(file_names)
  return command.send("ImageListAddFiles?", FileNames=file_names)

def index_of_file(file_name: str) -> int:
  return command.send("ImageListIndexOfFileName?", FileName=file_name)

def get_file_name(index = None) -> str:
  return command.send("ImageListGetFileName?", Index=index)

def remove_index(index: int):
  command.send("ImageListRemoveIndex", Index=index)

def remove_unchecked():
  command.send("ImageListRemoveUnchecked")

def remove_all():
  command.send("ImageListRemoveAll")

def clear():
  remove_all()

def uncheck_nonexistent():
  command.send("ImageListUncheckNonExistent")

def open_first(skip_save) -> bool:
  return command.send("ImageListOpenFirst?", SkipSave=skip_save)

def open_selected(skip_save):
  return command.send("ImageListOpenSelected", SkipSave=skip_save)

def open_next(skip_save, silent = True, can_cycle = False) -> bool:
  return command.send("ImageListOpenNext?", SkipSave=skip_save, Silent=silent, CanCycle=can_cycle)

def open_previous(skip_save, silent = True, can_cycle = False) -> bool:
  return command.send("ImageListOpenPrevious?", SkipSave=skip_save, Silent=silent, CanCycle=can_cycle)

def iterate(skip_save):
  if open_first(skip_save):
    yield get_file_name()
    while open_next(skip_save):
      yield get_file_name()

def get_file_checked(index = None) -> bool:
  return command.send("ImageListGetFileChecked?", Index=index)

def set_file_checked(index = None, checked = True) -> bool:
  return command.send("ImageListSetFileChecked", Index=index, Checked=checked)

def get_auto_uncheck_mode() -> str:
  return command.send("ImageListGetAutoUncheckMode?")

def set_auto_uncheck_mode(mode: str):
  command.send("ImageListSetAutoUncheckMode", Mode=mode)

def get_auto_zoom_fit() -> bool:
  return command.send("ImageListGetAutoZoomFit?")

def set_auto_zoom_fit(enabled: bool):
  command.send("ImageListSetAutoZoomFit", Enabled=enabled)




from lazpaint import command, dialog, colors, layer
import os

if __name__ == "__main__":
  dialog.show_message("Library to act on the whole image.")

RESAMPLE_QUALITY_BOX = 'Box'
RESAMPLE_QUALITY_LINEAR = 'Linear'
RESAMPLE_QUALITY_HALF_COSINE = 'HalfCosine'
RESAMPLE_QUALITY_COSINE = 'Cosine'
RESAMPLE_QUALITY_BICUBIC = 'Bicubic'
RESAMPLE_QUALITY_MITCHELL = 'Mitchell'
RESAMPLE_QUALITY_SPLINE = 'Spline'
RESAMPLE_QUALITY_LANCZOS2 = 'Lanczos2'
RESAMPLE_QUALITY_LANCZOS3 = 'Lanczos3'
RESAMPLE_QUALITY_LANCZOS4 = 'Lanczos4'
RESAMPLE_QUALITY_BEST = 'BestQuality'

ANCHOR_TOP_LEFT = 'TopLeft'
ANCHOR_TOP = 'Top'
ANCHOR_TOP_RIGHT = 'TopRight'
ANCHOR_LEFT = 'Left'
ANCHOR_MIDDLE = 'Middle'
ANCHOR_RIGHT = 'Right'
ANCHOR_BOTTOM_LEFT = 'BottomLeft'
ANCHOR_BOTTOM = 'Bottom'
ANCHOR_BOTTOM_RIGHT = 'BottomRight' 

def new(width: int, height: int, color=colors.TRANSPARENT, ignore_modified=False):
  command.send("FileNew", Width=width, Height=height, BackColor=color, IgnoreModified=ignore_modified)

def paste_as_new():
  command.send("EditPasteAsNew")

def get_width() -> int:
  return command.send("GetImageWidth?")

def get_height() -> int:
  return command.send("GetImageHeight?")

def get_size() -> tuple:
  return command.send("GetImageSize?")

def get_registry(identifier):
  str_result = command.send("ImageGetRegistry?", Identifier=identifier)
  if str_result == "":
    return None
  else:
    return command.parse_str(str_result)

def set_registry(identifier, value):
  if value == None:
    value = ""
  else:
    value = str(value)
  command.send("ImageSetRegistry", Identifier=identifier, Value=value)

def get_layer_index(layer_id=None) -> int:
  return command.send("GetLayerIndex?", LayerId=layer_id)

def iterate_layers():
  prev_id = layer.get_id()
  for layer_id in get_all_layers_id():
    layer.select_id(layer_id)
    yield layer_id
  layer.select_id(prev_id)

def get_all_layers_id() -> list:
  return command.send("GetAllLayersId?")

def contains_layer_id(layer_id) -> bool:
  return get_layer_index(layer_id) is not None

def select_layer_index(index: int): #1..layer_count
  return command.send("SelectLayerIndex", Index=index)

def move_layer_index(from_index: int, to_index: int):
  return command.send("ImageMoveLayerIndex", FromIndex=from_index, ToIndex=to_index)

def get_layer_count() -> int:
  return command.send("GetLayerCount?")

def get_frame_index() -> int: #1..frame_count
  return command.send("GetFrameIndex?")

def get_frame_count() -> int:
  return command.send("GetFrameCount?")

def load_frame(frame_index=None, ignore_modified=False) -> int:
  return command.send("FileChooseEntry?", EntryIndex=frame_index, IgnoreModified=ignore_modified)

def new_frame(width=None, height=None, back_color=colors.TRANSPARENT, ignore_modified=False) -> int:
  command.send("FileNewEntry", Width=width, Height=height, BackColor=back_color, IgnoreModified=ignore_modified)

def open(file_name=None, ignore_modified=False):
  command.send("FileOpen", FileName=file_name, IgnoreModified=ignore_modified)

def save(skip_options=False) -> str:
  return command.send("FileSave?", SkipOptions=skip_options)

def save_as(file_name=None, validate=False, overwrite=False, skip_options=False) -> str:
  return command.send("FileSaveAs?", FileName=file_name, Validate=validate, Overwrite=overwrite, SkipOptions=skip_options)

def export(file_name=None, validate=False, overwrite=False, skip_options=False) -> str:
  return command.send("FileSaveAs?", FileName=file_name, Validate=validate, Overwrite=overwrite, SkipOptions=skip_options, Export=True)

def change_file_extension(file_name: str, new_extension: str) -> str:
  base, ext = os.path.splitext(file_name)
  if len(new_extension) > 0 and new_extension[0:1] != ".":
    new_extension = "." + new_extension
  return base + new_extension

def get_temporary_name() -> str:
  return command.send("FileGetTemporaryName?")

def reload(ignore_modified=False):
  command.send("FileReload", IgnoreModified=ignore_modified)

def get_name() -> str:
  return command.send("GetFileName?")

def resample(width: int, height: int, quality=RESAMPLE_QUALITY_BEST):
  command.send("ImageResample", Width=width, Height=height, Quality=quality, Validate=True)

def smart_zoom3():
  command.send("ImageSmartZoom3")

def horizontal_flip():
  command.send("ImageHorizontalFlip")

def vertical_flip():
  command.send("ImageVerticalFlip")

def rotate_cw():
  command.send("ImageRotateCW")

def rotate_ccw():
  command.send("ImageRotateCCW")

def negative():
  command.send("ImageNegative")

def linear_negative():
  command.send("ImageLinearNegative")

def swap_red_blue():
  command.send("ImageSwapRedBlue")

def crop_to_selection():
  command.send("ImageCrop")

def crop_to_selection_and_layer():
  command.send("ImageCropLayer")

def flatten():
  command.send("ImageFlatten")

def canvas_size(width: int, height: int, anchor=ANCHOR_MIDDLE):
  command.send("ImageCanvasSize", Width=width, Height=height, Anchor=anchor)

def repeat(width: int, height: int, anchor=ANCHOR_MIDDLE, flip=False):
  command.send("ImageRepeat", Width=width, Height=height, Anchor=anchor, Flip=flip)

def undo():
  command.send("EditUndo")

def redo():
  command.send("EditRedo")

# starts a series of actions (undoable with only one call to "undo")
def do_begin():
  command.send("EditDoBegin")

# returns True if some action was done within the series of actions
def do_end() -> bool:
  return command.send("EditDoEnd?")




from lazpaint import image, layer, filters

for i in range(1, image.get_layer_count()+1):
  image.select_layer_index(i)
  filters.twirl(radius=min(image.get_width(),image.get_height())/2, angle=360)




from lazpaint import command

def zoom_in(fine=False):
  command.send("ViewZoomIn", Fine=fine)

def zoom_out(fine=False):
  command.send("ViewZoomOut", Fine=fine)

def zoom_fit():
  command.send("ViewZoomFit")

def set_zoom(factor=1.0):
  command.send("ViewZoomSet", Factor=factor)

def get_zoom() -> float:
  return command.send("ViewZoomGet?")

def set_grid_visible(visible=None):
  command.send("ViewGrid", Visible=visible)

def get_grid_visible() -> bool:
  return command.send("ViewGridGet?")



# Mask > Mask from alpha channel
# (fr) Masque > Masque depuis canal alpha
from lazpaint import image, layer, filters, selection, dialog

translation = dialog.translate_dict(["Mask"])

image.do_begin()

selection.deselect()
layer.duplicate()
layer.set_name(translation["Mask"])
filters.filter_function(red="alpha", green="alpha", blue="alpha", alpha=255, gamma_correction=False)
layer.set_blend_op(layer.BLEND_MASK)

image.do_end()



# Channels > Merge
# (fr) Canaux > Fusionner
from lazpaint import image, dialog, layer

translation = dialog.translate_dict(["Current layer is not split"])

channels_id = None
new_layer_index = None
image.do_begin()

if layer.get_registry("split-channel") is not None:
  layer_id = layer.get_registry("split-source-id")
  if image.contains_layer_id(layer_id):
    layer.select_id(layer_id)
    channels_id = layer.get_registry("split-channels-id")
    new_layer_index = image.get_layer_index() + 1
  else:
    channels_id = []
    for cur_layer_id in image.iterate_layers():
      cur_source_id = layer.get_registry("split-source-id")
      if cur_source_id == layer_id:
        channels_id.insert(0, cur_layer_id)   
        if new_layer_index is None:
          new_layer_index = image.get_layer_index()
else:
  layer_id = layer.get_id()
  channels_id = layer.get_registry("split-channels-id")
  new_layer_index = image.get_layer_index() + 1

if channels_id is None:
  dialog.show_message(translation["Current layer is not split"])
  exit()

layer.new(dialog.get_script_name())
image.move_layer_index(image.get_layer_index(), new_layer_index)

for cur_layer_id in reversed(channels_id):
  layer.select_id(cur_layer_id)
  image.move_layer_index(image.get_layer_index(), new_layer_index+1)
  layer.merge_over()

new_layer_id = layer.get_id()

if image.contains_layer_id(layer_id):
  layer.select_id(layer_id)
  layer_index = image.get_layer_index()
  layer_name = layer.get_name()
  layer_opacity = layer.get_opacity()
  image.move_layer_index(new_layer_index, layer_index+1)
  layer.select_id(layer_id)
  layer.remove()
  layer.select_id(new_layer_id)
  layer.set_name(layer_name)
  layer.set_opacity(layer_opacity)
  
image.do_end()



from lazpaint import filters, image

#filters.filter_function(green="green*2")
filters.phong(color_source=filters.PHONG_COLOR_LAYER, light_x_percent=20, light_y_percent=20, altitude_percent=30, altitude_source=filters.PHONG_ALTITUDE_ALPHA_CHANNEL)
#filters.emboss(angle=45, transparent=False, preserve_colors=True)
#filters.noise(grayscale=True, opacity=128)
filters.pixelate(20, filters.PIXELATE_QUALITY_BEST)
#filters.twirl(radius=min(image.get_width(),image.get_height())/2, angle=360)
#filters.blur_motion(50, 45, True)
#filters.sharpen(1)
filters.wave_displacement(100,30,0,25,25)
filters.rain(0.5, 0.5)



import ast
from lazpaint import command

def show_message(message):
  # wait for validation before continuing script
  command.send("ShowMessage?", Message=message)

def show_directory_dialog(prompt, initial_dir=None) -> str:
  return command.send("ShowDirectoryDialog?", Prompt=prompt, InitialDir=initial_dir)

def input_text(prompt, default=None) -> str:
  return command.send("InputBox?", Prompt=prompt, Default=default)

def input_value(prompt, default):
  return ast.literal_eval(input_text(prompt, str(default)))

def show_color_dialog(color=None) -> str:
  return command.send('ShowColorDialog?', Color=color)

def translate_text(text) -> str:
  return command.send('TranslateText?', Text=text)

def translate_dict(texts) -> dict:
  return dict(zip(texts, translate_text(texts)))

def get_language() -> str:
  return command.send('TranslateGetLanguage?')

def select_translation(**translations):
  lang = get_language()
  if translations.get(lang) is None:
    lang = "en"
  return translations[lang]

def get_script_name() -> str:
  return command.send('ScriptGetName?')



from lazpaint import image, layer, colors, view, dialog

dialog.show_message(image.get_size())
w = 256
h = 256
image.new(2, 2)
layer.put_image(0, 0, [[colors.RGB(128,128,255), colors.RGB(0,255,255)], [colors.RGB(255,0,255), colors.RGB(255,255,255)]], layer.DM_SET)
image.resample(w, h)
image.repeat(w*4, h*4, anchor=image.ANCHOR_TOP_LEFT)
view.zoom_fit()
pix1 = layer.get_pixel(0,0)

image.horizontal_flip()
assert layer.get_pixel(w-1,0) == pix1  
image.vertical_flip()
assert layer.get_pixel(w-1,h-1) == pix1  
image.rotate_cw()
assert layer.get_pixel(0,h-1) == pix1  
image.rotate_cw()
assert layer.get_pixel(0,0) == pix1  
image.linear_negative()
pix1 = pix1.linear_negative()
assert layer.get_pixel(0,0) == pix1
pix1 = pix1.swap_red_blue()
image.swap_red_blue()
assert layer.get_pixel(0,0) == pix1

layer.new()
layer.fill(colors.RGBA(192,192,192,64))
image.flatten()



from lazpaint import command, dialog

if __name__ == "__main__":
  dialog.show_message("Library to execute filters on the current layer.")

#filters
BLUR_PRECISE = 'BlurPrecise'
BLUR_RADIAL = 'BlurRadial'
BLUR_FAST = 'BlurFast'
BLUR_BOX = 'BlurBox'
BLUR_CORONA = 'BlurCorona'
BLUR_DISK = 'BlurDisk'
BLUR_MOTION = 'BlurMotion'
BLUR_CUSTOM = 'BlurCustom'
PIXELATE = 'Pixelate'

SHARPEN = 'Sharpen'
SMOOTH = 'Smooth'
MEDIAN = 'Median'
NOISE = 'Noise'
CLEAR_TYPE = 'ClearType'
CLEAR_TYPE_INVERSE = 'ClearTypeInverse'
FILTER_FUNCTION = 'Function'
CONTOUR = 'Contour'
EMBOSS = 'Emboss'
PHONG = 'Phong'

SPHERE = 'Sphere'
TWIRL = 'Twirl'
WAVE_DISPLACEMENT = 'WaveDisplacement'
CYLINDER = 'Cylinder'
PLANE = 'Plane'

PIXELATE_QUALITY_FAST = 'Fast'
PIXELATE_QUALITY_LINEAR = 'Linear'
PIXELATE_QUALITY_MITCHELL = 'Mitchell'
PIXELATE_QUALITY_SPLINE = 'Spline'
PIXELATE_QUALITY_BEST = PIXELATE_QUALITY_MITCHELL

PHONG_COLOR_LAYER = 'Layer'
PHONG_COLOR_PEN = 'Pen'
PHONG_COLOR_BACK = 'Back'

PHONG_ALTITUDE_LIGHTNESS = 'Lightness'
PHONG_ALTITUDE_LINEAR_LIGHTNESS = 'LinearLightness'
PHONG_ALTITUDE_SATURATION = 'Saturation'
PHONG_ALTITUDE_ALPHA_CHANNEL = 'Alpha'
PHONG_ALTITUDE_RED_CHANNEL = 'Red'
PHONG_ALTITUDE_GREEN_CHANNEL = 'Green'
PHONG_ALTITUDE_BLUE_CHANNEL = 'Blue'

#colors
COLOR_COMPLEMENTARY = 'ComplementaryColor'
COLOR_NEGATIVE = 'Negative'
COLOR_LINEAR_NEGATIVE = 'LinearNegative'
COLOR_NORMALIZE = 'Normalize'
COLOR_GRAYSCALE = 'Grayscale'

#render
RENDER_PERLIN_NOISE = 'PerlinNoise'
RENDER_CYCLIC_PERLIN_NOISE = 'CyclicPerlinNoise'
RENDER_CLOUDS = 'Clouds'
RENDER_CUSTOM_WATER = 'CustomWater'
RENDER_WATER = 'Water'
RENDER_RAIN = 'Rain'
RENDER_WOOD = 'Wood'
RENDER_WOOD_VERTICAL = 'WoodVertical'
RENDER_PLASTIK = 'Plastik'
RENDER_METAL_FLOOR = 'MetalFloor'
RENDER_CAMOUFLAGE = 'Camouflage'
RENDER_SNOW_PRINT = 'SnowPrint'
RENDER_STONE = 'Stone'
RENDER_ROUND_STONE = 'RoundStone'
RENDER_MARBLE = 'Marble'

def run(name, validate=True):
  if name[0:5] == "Color":
    command.send(name, Validate=validate)
  else:
    command.send("Filter", Name=name, Validate=validate)

def blur(name=BLUR_FAST, radius=None, validate=True): #radius: float or (x,y) 
  if isinstance(radius, tuple):
    radius_x = radius[0]
    radius_y = radius[1]
    radius = None
  else:
    radius_x = None
    radius_y = None
  command.send("Filter", Name=name, Radius=radius, RadiusX=radius_x, RadiusY=radius_y, Validate=validate)

def blur_motion(distance=None, angle=None, oriented=None, validate=True): #oriented: bool
  command.send("Filter", Name=BLUR_MOTION, Distance=distance, Angle=angle, Oriented=oriented, Validate=validate)

def sharpen(amount=None, validate=True): #amout: 0..10
  command.send("Filter", Name=SHARPEN, Amount=amount, Validate=validate)

def noise(grayscale=None, opacity=None, validate=True): #grayscale: bool, opacity: 0..255
  command.send("Filter", Name=NOISE, Grayscale=grayscale, Opacity=opacity, Validate=validate)

def pixelate(pixel_size=None, quality=None, validate=True):
  command.send("Filter", Name=PIXELATE, PixelSize=pixel_size, Quality=quality, Validate=validate)

def filter_function(red=None, green=None, blue=None, alpha=None, hue=None, saturation=None, lightness=None, L=None, a=None, b=None, corrected_hue=None, gamma_correction=None, validate=True): #expressions: str
  command.send("Filter", Name=FILTER_FUNCTION, Red=red, Green=green, Blue=blue, Alpha=alpha, Hue=hue, Saturation=saturation, Lightness=lightness, CorrectedHue=corrected_hue, GammaCorrection=gamma_correction, Validate=validate)

def emboss(angle=None, transparent=None, preserve_colors=None, validate=True):
  command.send("Filter", Name=EMBOSS, Angle=angle, Transparent=transparent, PreserveColors=preserve_colors, Validate=validate)

def rain(amount=None, wind=None, validate=True): #amount and wind: 0..2
  command.send("Filter", Name=RENDER_RAIN, Amount=amount, Wind=wind, Validate=validate)

def phong(color_source=None, altitude_percent=None, altitude_source=None, light_pos_percent=None, light_x_percent=None, light_y_percent=None, validate=True):  
  command.send("Filter", Name=PHONG, ColorSource=color_source, AltitudePercent=altitude_percent, AltitudeSource=altitude_source, LightPosPercent=light_pos_percent, LightXPercent=light_x_percent, LightYPercent=light_y_percent, Validate=validate)

def twirl(radius=None, angle=None, center_pos_percent=None, center_x_percent=None, center_y_percent=None, validate=True):
  command.send("Filter", Name=TWIRL, Radius=radius, Angle=angle, CenterPosPercent=center_pos_percent, CenterXPercent=center_x_percent, CenterYPercent=center_y_percent, Validate=validate)

def wave_displacement(wave_length=None, displacement=None, phase=None, center_pos_percent=None, center_x_percent=None, center_y_percent=None, validate=True): #phase: 0..360
  command.send("Filter", Name=WAVE_DISPLACEMENT, WaveLength=wave_length, Displacement=displacement, Phase=phase, CenterPosPercent=center_pos_percent, CenterXPercent=center_x_percent, CenterYPercent=center_y_percent, Validate=validate)



# Layer effect > Color overlay
# (fr) Effet de calque > Couleur superposée
from lazpaint import image, colors, layer, filters

color = colors.show_dialog(layer.get_registry("overlay-color"))
if color is not None:
  image.do_begin()
  filters.filter_function(red = color.red/255, green = color.green/255, blue = color.blue/255, gamma_correction = False)
  layer.set_registry("overlay-color", color)
  image.do_end()




# Channels > Split CMYK
# (fr) Canaux > Séparer CMJN
from lazpaint import image, dialog, layer, filters

translation = dialog.translate_dict(["Layer already split", "Cyan", "Magenta", "Yellow", "Black", "Alpha"])

# check if it is a channel
if layer.get_registry("split-channel") is not None:
  dialog.show_message(translation["Layer already split"])
  exit()

layer_id = layer.get_id()
layer_index = image.get_layer_index()
layer_opacity = layer.get_opacity()
layer_transparent = layer.is_transparent()
cmy_id = None
black_id = None

# check if it has been split
if layer.get_registry("split-channels-id") is not None:
  for cur_layer_id in image.iterate_layers():
    if layer.get_registry("split-source-id") == layer_id:
      dialog.show_message(translation["Layer already split"])
      exit()

image.do_begin()
channels = []
if layer_transparent:
  channels.append({"name": "Alpha", "channel": "A", "red": "alpha", "green": "alpha", "blue": "alpha", "alpha": "255"})
channels.append({"name": "Black", "channel": "K", "red":"max(max(red,green),blue)", "green": "max(max(red,green),blue)", "blue": "max(max(red,green),blue)",  "alpha": "255"})
#channels.append({"name": "Black", "channel": "K", "hue":"0", "saturation": "0", "lightness": "max(max(red,green),blue)",  "alpha": "255"})

channels_id = [] 
for ch in channels:
  layer.select_id(layer_id)
  layer.duplicate()
  filters.filter_function(red = ch.get("red"), green = ch.get("green"), blue = ch.get("blue"), hue = ch.get("hue"), saturation = ch.get("saturation"), lightness = ch.get("lightness"), alpha = ch["alpha"], gamma_correction = False, corrected_hue = False)
  layer.set_name(translation[ch["name"]])
  layer.set_opacity(layer_opacity)
  if ch["channel"] == "A":
    layer.set_blend_op(layer.BLEND_MASK)
  layer.set_registry("split-channel", ch["channel"])
  layer.set_registry("split-source-id", layer_id)
  if ch["channel"] == "K":
    black_id = layer.get_id()
    layer.duplicate()
    layer.set_blend_op(layer.BLEND_DIVIDE)
    black_div_id = layer.get_id()
    layer.select_id(layer_id)
    layer.duplicate()
    copy_index = image.get_layer_index()
    image.move_layer_index(copy_index, copy_index + 1)
    image.select_layer_index(copy_index + 2)
    layer.merge_over()
    cmy_id = layer.get_id()
  else:
    channels_id.append(layer.get_id())

if cmy_id is not None:
  channels = []
  channels.append({"name": "Cyan", "channel": "C", "red": "red", "green": "1", "blue": "1", "alpha": "255"})
  channels.append({"name": "Magenta", "channel": "M", "red": "1", "green": "green", "blue": "1", "alpha": "255"})
  channels.append({"name": "Yellow", "channel": "Y", "red": "1", "green": "1", "blue": "blue", "alpha": "255"})
  channels[-1]["last"] = True
  for ch in channels:
    layer.select_id(cmy_id)
    if ch.get("last") is None:
      layer.duplicate()
    filters.filter_function(red = ch.get("red"), green = ch.get("green"), blue = ch.get("blue"), alpha = ch["alpha"], gamma_correction = False)
    layer.set_name(translation[ch["name"]])
    layer.set_opacity(layer_opacity)
    layer.set_blend_op(layer.BLEND_MULTIPLY)
    layer.set_registry("split-channel", ch["channel"])
    layer.set_registry("split-source-id", layer_id)
    channels_id.append(layer.get_id())
    
if black_id is not None:
  channels_id.append(black_id)
  
layer.select_id(layer_id)
layer.set_registry("split-channels-id", channels_id)
layer.set_visible(False)
image.do_end()



# Negative
from lazpaint import image, layer, colors, dialog

width, height = image.get_size()
image = layer.get_image(0, 0, width, height)
for y in range(0, height):
  scanline = image[y]
  for x in range(0, width):    
    scanline[x] = scanline[x].linear_negative()
layer.put_image(0, 0, image, layer.DM_SET)




from lazpaint import tools, image, layer, dialog

def line(x, y, x2, y2):
    tools.choose(tools.PEN)
    tools.mouse([(x, y), (x2, y2)])

ZOOM = dialog.input_value("Zoom (Between 0.1 and 0.4 it creates a 3d room, more than 0.5 to 0.9 it creates a cross with a rectangle)", 0.25)

image.do_begin()
layer.new()
w = image.get_width()
h = image.get_height() 

#top left
line(0, 0, w * ZOOM, h * ZOOM)
#bottom left
line(0, h, w * ZOOM, h - (h * ZOOM))
#top right
line(w, 0, w - (w * ZOOM), h * ZOOM)
#bottom right
line(w, h, w - (w * ZOOM), h - (h * ZOOM))
#top
line(w * ZOOM, h * ZOOM, w - (w * ZOOM),  h * ZOOM)
#bottom
line(w * ZOOM, h - (h * ZOOM), w - (w * ZOOM), h - (h * ZOOM))
#left
line(w * ZOOM, h * ZOOM, w * ZOOM, h - (h * ZOOM))
#right
line(w - (w * ZOOM), h - (h * ZOOM), w - (w * ZOOM), h * ZOOM)

image.do_end()



from lazpaint import layer, dialog, colors

layer_id = layer.get_id()
layer.duplicate()
layer.horizontal_flip()
layer.duplicate()
layer.vertical_flip()
layer.merge_over()
layer.fill_background(colors.RED)
layer.select_id(layer_id)
layer.remove()



from lazpaint import image, layer, dialog, colors, view

view.set_zoom(1)
width = 256
height = 256
image.new(width, height)
red = dialog.input_value("Red value (0..255)", 0)
image = []
for y in range(height):
  scanline = [colors.RGB(red,x,y) for x in range(width)]
  image.append(scanline)

layer.put_image(0, 0, image, layer.DM_SET)
if layer.get_pixel(192,64).green != 192:
  dialog.show_message("The value of the pixel is not correct.")
else:
  dialog.show_message("Test successful.")
  
view.zoom_fit()
view.set_zoom(10)
view.set_grid_visible(True)



from lazpaint import image, colors, layer, selection, dialog

current_file_name = image.get_name()
if current_file_name is not None: 
  dialog.show_message("Filename is \"" + current_file_name + "\"")
else:
  dialog.show_message("Image doesn't have a filename")  

if selection.is_mask_empty():
  selection_name = None
  dialog.show_message("There is no selection mask")
else: 
  selection_name = selection.save_as("script_test_selection.png")
  dialog.show_message("Selection saved")

image.new(100, 100, colors.RED)

if selection_name is not None:
  selection.load(selection_name)
  dialog.show_message("Selection restored")

wanted_file_name = dialog.input_text("Test file name:", "script_test_file.png")
file_name = image.save_as(wanted_file_name, skip_options=True)
image.new(100, 100, colors.LIME)
image.save_as(file_name, validate=True, overwrite=True, skip_options=True)

layer.fill(colors.BLUE)
image.reload(ignore_modified=True)

layer.fill(colors.PURPLE)
image.save(skip_options=True)



from lazpaint import command, dialog, colors

if __name__ == "__main__":
  dialog.show_message("Library to access layer content.")

DM_DRAW = "dmDrawWithTransparency"
DM_LINEAR = "dmLinearBlend"
DM_SET = "dmSet"
DM_SET_EXCEPT_TRANSPARENT = "dmSetExceptTransparent"
DM_XOR = "dmXor"

BLEND_DRAW = 'Transparent'
BLEND_LINEAR = 'LinearBlend'
BLEND_LIGHTEN = 'Lighten'
BLEND_SCREEN = 'Screen'
BLEND_ADD = 'Additive'
BLEND_LINEAR_ADD = 'LinearAdd'
BLEND_COLOR_DODGE = 'ColorDodge'
BLEND_DIVIDE = 'Divide'
BLEND_NICE_GLOW = 'NiceGlow'
BLEND_SOFT_LIGHT = 'SoftLight'
BLEND_HARD_LIGHT = 'HardLight'
BLEND_GLOW = 'Glow'
BLEND_REFLECT = 'Reflect'
BLEND_OVERLAY = 'Overlay'
BLEND_DARK_OVERLAY = 'DarkOverlay'
BLEND_DARKEN = 'Darken'
BLEND_MULTIPLY = 'Multiply'
BLEND_COLOR_BURN = 'ColorBurn'
BLEND_DIFFERENCE = 'Difference'
BLEND_LINEAR_DIFFERENCE = 'LinearDifference'
BLEND_EXCLUSION = 'Exclusion'
BLEND_LINEAR_EXCLUSION = 'LinearExclusion'
BLEND_SUBTRACT = 'Subtract'
BLEND_LINEAR_SUBTRACT = 'LinearSubtract'
BLEND_SUBTRACT_INVERSE = 'SubtractInverse'
BLEND_LINEAR_SUBTRACT_INVERSE = 'LinearSubtractInverse'
BLEND_NEGATION = 'Negation'
BLEND_LINEAR_NEGATION = 'LinearNegation'
BLEND_XOR = 'Xor'
BLEND_SVG_SOFT_LIGHT = 'SvgSoftLight'
BLEND_MASK = 'Mask'
BLEND_LINEAR_MULTIPLY_SATURATION = 'LinearMultiplySaturation'
BLEND_LINEAR_HUE = 'LinearHue'
BLEND_LINEAR_COLOR = 'LinearColor'
BLEND_LINEAR_LIGHTNESS = 'LinearLightness'
BLEND_LINEAR_SATURATION = 'LinearSaturation'
BLEND_CORRECTED_HUE = 'CorrectedHue'
BLEND_CORRECTED_COLOR = 'CorrectedColor'
BLEND_CORRECTED_LIGHTNESS = 'CorrectedLightness'
BLEND_CORRECTED_SATURATION = 'CorrectedSaturation'

def get_id():
  return command.send("LayerGetId?")

def get_name() -> str:
  return command.send("LayerGetName?")

def get_opacity() -> int:
  return command.send("LayerGetOpacity?")

def get_blend_op():
  return command.send("LayerGetBlendOp?")

def get_visible() -> bool:
  return command.send("LayerGetVisible?")

def select_id(id):
  command.send("LayerSelectId", Id=id)

def set_name(name: str):
  return command.send("LayerSetName", Name=str(name))

def set_opacity(opacity: int):
  return command.send("LayerSetOpacity", Opacity=opacity)

def set_blend_op(blend_op):
  return command.send("LayerSetBlendOp", BlendOp=blend_op)

def set_visible(visible: bool):
  return command.send("LayerSetVisible", Visible=visible)

def new(name: str = None): #-> id
  layer_id = command.send("LayerAddNew?")
  if name is not None:
    set_name(name)
  return layer_id

def paste_as_new(): #-> id
  return command.send("EditPasteAsNewLayer?")

def add_from_file(file_name): #-> [id]
  return command.send("LayerFromFile?", FileName=file_name)

def save_as(file_name:str, format:str = None): #-> str
  return command.send("LayerSaveAs?", FileName=file_name, Format=format)

def duplicate(): #-> id
  return command.send("LayerDuplicate?")

def merge_over():
  command.send("LayerMergeOver")

def is_empty() -> bool:
  return command.send("IsLayerEmpty?")

def is_transparent() -> bool:
  return command.send("IsLayerTransparent?")

def get_registry(identifier):
  str_result = command.send("LayerGetRegistry?", Identifier=identifier)
  if str_result == "":
    return None
  else:
    return command.parse_str(str_result)

def set_registry(identifier, value):
  if value == None:
    value = ""
  elif isinstance(value, str):
    value = repr(value)
  command.send("LayerSetRegistry", Identifier=identifier, Value=value)

def remove():
  command.send("LayerRemoveCurrent")

def get_count() -> int:
  return command.send("GetLayerCount?")

def rasterize():
  command.send("LayerRasterize")

def get_image_width(image) -> int:
  return max([len(scanline) for scanline in image])

def get_image_height(image) -> int:
  return len(image)

def get_image_size(image):
  height = get_image_height(image)
  if height == 0:
    return (0,0)
  else:
    return (get_image_width(image), height)

def put_image(x: int, y: int, image, mode=DM_DRAW, opacity=255):
  width, height = get_image_size(image)
  if width == 0 or height == 0: return
  flattened = ""
  for scanline in image:
    flattened += "".join([str(color) for color in scanline]) + "00000000" * (width - len(scanline))
  command.send("PutImage", X=x, Y=y, Width=width, Height=height, Data=flattened, Mode=mode, Opacity=opacity)

def get_image(x: int, y: int, width: int, height: int):
  flattened = command.send("GetImage?", X=x, Y=y, Width=width, Height=height)
  str_pos = 0
  image = []
  for yb in range(0, height):
    scanline = []
    for xb in range(0, width):       
      scanline.append(colors.RGBA(int(flattened[str_pos:str_pos + 2],16), int(flattened[str_pos + 2:str_pos + 4],16), int(flattened[str_pos + 4:str_pos + 6],16), int(flattened[str_pos + 6:str_pos + 8],16)))
      str_pos = str_pos + 8
    image.append(scanline)
  return image    

def get_pixel(x: int, y: int):
  return colors.str_to_RGBA(command.send("GetPixel?", X=x, Y=y))

def fill(color, mode=DM_DRAW):
  command.send("LayerFill", Color=color, Mode=mode)

def horizontal_flip():
  command.send("LayerHorizontalFlip")

def vertical_flip():
  command.send("LayerVerticalFlip")

def clear_alpha(back_color=None):
  command.send("ImageClearAlpha", BackColor=back_color)

def fill_background(back_color=None):
  command.send("ImageFillBackground", BackColor=back_color)



import ast

# tells LazPaint it is a script, we are going to send commands
print("LazPaint script\t")
# wait for LazPaint response
if input('') != chr(27) + 'LazPaint': 
  print("Needs to be run from LazPaint.")
  exit()
  
import sys
if sys.platform == "win32":
  import io
  sys.stdout = io.TextIOWrapper(sys.stdout.detach(), encoding = 'utf-8')
  sys.stderr = io.TextIOWrapper(sys.stderr.detach(), encoding = 'utf-8')
  sys.stdin = io.TextIOWrapper(sys.stdin.detach(), encoding = 'utf-8')

def parse_str(text: str):
  if text[:1] == "#":
    return text
  else:
    return ast.literal_eval(text)

# sends a command to LazPaint
def send(command: str, **keywords):
  if keywords is None:
    print(chr(27) + command)
  else:
    print(chr(27) + command + chr(29) + repr(keywords))
  if command[-1] == '?':
    return parse_str(input(''))
  else:
    return

def get_version(): # (major, minor, revision)
  return send("LazPaintGetVersion?")



# Layer effect > Stroke
# (fr) Effet de calque > Trait
from lazpaint import dialog

try:
    from tkinter import *
except ImportError:
    dialog.show_message("Please install tkinter. On Debian distributions, use the command: apt install python3-tk")
    exit()
        
from lazpaint import colors, image, layer, filters, tools, selection

translation = dialog.translate_dict(["Layer is empty", "Radius", "Color", "Opacity", "Ok", "Cancel"])

if layer.is_empty():
    dialog.show_message(translation["Layer is empty"])
    exit()

############ image processing

FRIENDLY_NAME = dialog.get_script_name()
MAX_RADIUS = 100
MAX_OPACITY = 255

source_layer_id = layer.get_id()
source_layer_name = layer.get_name()

chosen_radius = layer.get_registry("stroke-radius")
if chosen_radius == None: 
    chosen_radius = image.get_registry("stroke-radius")
if chosen_radius == None:
    chosen_radius = 10

stroke_layer_id = layer.get_registry("stroke-layer-id")
if image.get_layer_index(stroke_layer_id) == None:
    stroke_layer_id = None

if stroke_layer_id is not None:
    layer.select_id(stroke_layer_id)
    chosen_opacity = layer.get_opacity()
    overlay_color = colors.str_to_RGBA(layer.get_registry("overlay-color"))
    layer.select_id(source_layer_id)
else:
    chosen_opacity = layer.get_opacity() 
    overlay_color = None

if overlay_color is None:
  overlay_color = colors.BLACK

def create_stroke_layer():
    global stroke_layer_id
    image.do_begin()
    if stroke_layer_id != None:
        layer.select_id(stroke_layer_id)
        stroke_index = image.get_layer_index()
        selection.select_all()
        selection.delete()
        layer.select_id(source_layer_id)
        layer.duplicate()
        image.move_layer_index(image.get_layer_index(), stroke_index+1)
        layer.merge_over()        
    else:
        layer.select_id(source_layer_id)
        layer.duplicate()
        layer.set_name(FRIENDLY_NAME + " - " + source_layer_name)
        layer.set_registry("stroke-source-layer-id", source_layer_id)
        stroke_layer_id = layer.get_id()
        stroke_index = image.get_layer_index()
        image.move_layer_index(stroke_index, stroke_index-1)
    image.do_end()

stroke_done = False
opacity_done = False
stroke_initial_color = None

def apply_stroke():
    global stroke_done, opacity_done, stroke_initial_color
    if opacity_done:
        image.undo()
        opacity_done = False
    if stroke_done:
        image.undo()
        stroke_done = False
    image.do_begin() 
    if chosen_radius > 0.5:
        layer.duplicate()
        disk_id = layer.get_id()
        filters.blur(name=filters.BLUR_DISK, radius=chosen_radius-0.5)
        filters.filter_function(red=overlay_color.red/255, green=overlay_color.green/255, blue=overlay_color.blue/255, alpha="min(alpha*"+str(chosen_radius*5)+",(1-alpha)*"+str(chosen_radius*5)+")", gamma_correction=False)
        layer.select_id(stroke_layer_id) 
    else:
        disk_id = None   
    filters.blur(name=filters.BLUR_CORONA, radius=chosen_radius+0.5)
    filters.filter_function(red=overlay_color.red/255, green=overlay_color.green/255, blue=overlay_color.blue/255, alpha="min(alpha*"+str(chosen_radius)+",(1-alpha)*"+str(chosen_radius)+")", gamma_correction=False)
    if disk_id is not None:
        layer.select_id(disk_id)
        layer.merge_over()
    layer.set_registry("overlay-color", overlay_color)
    stroke_initial_color = overlay_color
    stroke_done = image.do_end()
    apply_opacity()

def apply_opacity():
    global opacity_done, chosen_opacity, overlay_color
    if opacity_done:
        image.undo()
        opacity_done = False
    image.do_begin()
    layer.set_opacity(chosen_opacity)
    if overlay_color != stroke_initial_color:
        filters.filter_function(red=overlay_color.red/255, green=overlay_color.green/255, blue=overlay_color.blue/255, gamma_correction=False)
        layer.set_registry("overlay-color", overlay_color)
    opacity_done = image.do_end()

######## interface

def button_ok_click():
    global source_layer_id, chosen_radius, chosen_offset
    layer.select_id(source_layer_id)
    layer.set_registry("stroke-radius", chosen_radius)
    layer.set_registry("stroke-layer-id", stroke_layer_id)
    image.set_registry("stroke-radius", chosen_radius)
    image.do_end()
    exit()

def button_cancel_click():    
    if image.do_end():
        image.undo()
    layer.select_id(source_layer_id)
    exit()

scale_radius_update_job = None

def scale_radius_update_do():
    global scale_radius_update_job, chosen_radius, scale_radius
    new_radius = scale_radius.get() 
    if new_radius != chosen_radius:
        chosen_radius = new_radius
        apply_stroke()
    scale_radius_update_job = None    

def scale_radius_update(event):
    global window, scale_radius_update_job
    if scale_radius_update_job:
        window.after_cancel(scale_radius_update_job)
    scale_radius_update_job = window.after(500, scale_radius_update_do)

scale_opacity_update_job = None

def scale_opacity_update_do():
    global chosen_opacity 
    new_opacity = scale_opacity.get()
    if new_opacity != chosen_opacity:
        chosen_opacity = new_opacity
        apply_opacity()
    scale_opacity_update_job = None

def scale_opacity_update(event):   
    global window, scale_opacity_update_job
    if scale_opacity_update_job:
        window.after_cancel(scale_opacity_update_job)
    scale_opacity_update_job = window.after(100, scale_opacity_update_do)

def button_color_click():
    global overlay_color, window
    new_color = colors.show_dialog(overlay_color)
    window.attributes('-topmost', True)
    window.attributes('-topmost', False)
    if new_color is not None and new_color != overlay_color:
        overlay_color = new_color
        apply_opacity()

window = Tk()
window.title(FRIENDLY_NAME)
window.resizable(False, False)

frame = Frame(window)
frame.pack()

label_radius = Label(frame, text=translation["Radius"])
label_radius.grid(column=0, row=0)
scale_radius = Scale(frame, from_=0, to=MAX_RADIUS, orient=HORIZONTAL, command=scale_radius_update)
scale_radius.grid(column=1, row=0, sticky=W+E, padx=10)
scale_radius.set(chosen_radius)

label_opacity = Label(frame, text=translation["Opacity"])
label_opacity.grid(column=0, row=1)
scale_opacity = Scale(frame, from_=0, to=MAX_OPACITY, orient=HORIZONTAL, command=scale_opacity_update)
scale_opacity.grid(column=1, row=1, sticky=W+E, padx=10)
scale_opacity.set(chosen_opacity)

label_color = Label(frame, text=translation["Color"])
label_color.grid(column=0, row=2)
button_color = Button(frame, text=translation["Color"] + "...", command=button_color_click)
button_color.grid(column=1, row=2)

frame.columnconfigure(0, pad=20)
frame.columnconfigure(1, minsize=250)
frame.rowconfigure(0, pad=20)
frame.rowconfigure(1, pad=20)
frame.rowconfigure(2, pad=20)

button_ok = Button(window, text=translation["Ok"], command=button_ok_click)
button_ok.pack(side=RIGHT, padx=10, pady=10)
button_cancel = Button(window, text=translation["Cancel"], command=button_cancel_click)
button_cancel.pack(side=RIGHT, pady=10)

image.do_begin()
selection.deselect()
create_stroke_layer()
apply_stroke()

window.update()
window_width = window.winfo_width()
screen_width = window.winfo_screenwidth()
window.geometry('+%d+0' % (int((screen_width - window_width) / 2)))

window.lift()
window.attributes('-topmost',True)
window.after_idle(window.attributes,'-topmost',False)

window.mainloop()
button_cancel_click()



# Render > Lava
# (fr) Rendu > Lave
# (es) Renderizar > Lava
# (de) Rendering > Lava
from lazpaint import image, layer, filters, colors, dialog

image.do_begin()
layer.new()
filters.run(filters.RENDER_WOOD)
layer.new()
filters.run(filters.RENDER_WATER)
layer.set_blend_op(layer.BLEND_LINEAR_SUBTRACT)
layer.merge_over()
layer.set_name(dialog.get_script_name())
image.do_end()




# Layer effect > Inner shadow
# (fr) Effet de calque > Ombre intérieure
from lazpaint import dialog

try:
    from tkinter import *
except ImportError:
    dialog.show_message("Please install tkinter. On Debian distributions, use the command: apt install python3-tk")
    exit()
        
from lazpaint import colors, image, layer, filters, tools, selection
import math

translation = dialog.translate_dict(["Layer is empty", "Radius", "Angle", "Opacity", "Ok", "Cancel"])

if layer.is_empty():
    dialog.show_message(translation["Layer is empty"])
    exit()

############ image processing

FRIENDLY_NAME = dialog.get_script_name()
REGISTRY_NAME = "innershadow"
OPPOSITE_REGISTRY_NAME = "innerlight"
DEFAULT_ANGLE = 135
DEFAULT_COLOR = colors.BLACK

MAX_RADIUS = 200
MAX_OPACITY = 255
MAX_ANGLE = 360

source_layer_id = layer.get_registry(REGISTRY_NAME+"-source-layer-id")
if source_layer_id is not None:
  layer.select_id(source_layer_id)
else:
  source_layer_id = layer.get_id()
source_layer_name = layer.get_name()

chosen_radius = layer.get_registry(REGISTRY_NAME+"-radius")
if chosen_radius == None: 
    chosen_radius = layer.get_registry(OPPOSITE_REGISTRY_NAME+"-radius")
if chosen_radius == None: 
    chosen_radius = image.get_registry(REGISTRY_NAME+"-radius")
if chosen_radius == None: 
    chosen_radius = image.get_registry(OPPOSITE_REGISTRY_NAME+"-radius")
if chosen_radius == None:
    chosen_radius = 20

chosen_angle = layer.get_registry(REGISTRY_NAME+"-angle")
if chosen_angle == None: 
    chosen_angle = layer.get_registry(OPPOSITE_REGISTRY_NAME+"-angle")
    if chosen_angle is not None:
        chosen_angle = (chosen_angle+180) % 360
if chosen_angle == None: 
    chosen_angle = image.get_registry(REGISTRY_NAME+"-angle")
if chosen_angle == None: 
    chosen_angle = image.get_registry(OPPOSITE_REGISTRY_NAME+"-angle")
    if chosen_angle is not None:
        chosen_angle = (chosen_angle+180) % 360
if chosen_angle == None:
    chosen_angle = DEFAULT_ANGLE

shadow_layer_id = layer.get_registry(REGISTRY_NAME+"-layer-id")
if image.get_layer_index(shadow_layer_id) == None:
    shadow_layer_id = None

if shadow_layer_id is not None:
    layer.select_id(shadow_layer_id)
    chosen_opacity = layer.get_opacity()
    overlay_color = colors.str_to_RGBA(layer.get_registry("overlay-color"))
    layer.select_id(source_layer_id)
else:
    chosen_opacity = layer.get_opacity()*2/3 
    overlay_color = None

if overlay_color is None:
  overlay_color = DEFAULT_COLOR

def create_shadow_layer():
    global shadow_layer_id
    image.do_begin()
    if shadow_layer_id != None:
        layer.select_id(shadow_layer_id)
        layer.remove()
    layer.select_id(source_layer_id)
    layer.duplicate()
    layer.rasterize()
    layer.set_name(FRIENDLY_NAME+" - "+source_layer_name)
    layer.set_registry(REGISTRY_NAME+"-source-layer-id", source_layer_id)
    shadow_layer_id = layer.get_id()
    layer.set_registry("overlay-color", overlay_color)
    layer.set_opacity(chosen_opacity)    
    image.do_end()

blur_done = False
opacity_done = False

def apply_blur():
    global blur_done, opacity_done
    if opacity_done:
        image.undo()
        opacity_done = False
    if blur_done:
        image.undo()
        blur_done = False
    image.do_begin()
    if chosen_radius == 0:
        filters.filter_function(alpha=0, gamma_correction=False)
    else:
        layer.duplicate()
        layer.set_opacity(255)
        filters.filter_function(red="alpha", green="alpha", blue="alpha", alpha=1, gamma_correction=False)
        mask_layer_id = layer.get_id()    

        layer.select_id(shadow_layer_id)
        filters.filter_function(red=overlay_color.red/255, green=overlay_color.green/255, blue=overlay_color.blue/255, gamma_correction=False)
        tools.choose(tools.MOVE_LAYER)
        offset = (math.sin(chosen_angle*math.pi/180)*chosen_radius, -math.cos(chosen_angle*math.pi/180)*chosen_radius)
        tools.mouse([(0,0), (offset[0],offset[1])])

        layer.select_id(mask_layer_id)
        layer.duplicate()
        mask_layer_id2 = layer.get_id()
        layer.select_id(mask_layer_id)
        tools.choose(tools.MOVE_LAYER)
        tools.mouse([(offset[0]/2,offset[1]/2), (0,0)])
        colors.linear_negative()
        layer.set_blend_op(layer.BLEND_MASK)
        layer.merge_over()
        mask_layer_id = mask_layer_id2

        filters.blur(radius=chosen_radius)

        layer.select_id(mask_layer_id)
        layer.set_blend_op(layer.BLEND_MASK)
        layer.merge_over()  
   
    blur_done = image.do_end()
    apply_opacity()

def apply_opacity():
    global opacity_done
    if opacity_done:
        image.undo()
        opacity_done = False
    image.do_begin()
    layer.set_opacity(chosen_opacity)
    opacity_done = image.do_end()

######## interface

def button_ok_click():
    global source_layer_id, chosen_radius, chosen_angle
    layer.select_id(source_layer_id)
    layer.set_registry(REGISTRY_NAME+"-radius", chosen_radius)
    layer.set_registry(REGISTRY_NAME+"-angle", chosen_angle)
    layer.set_registry(REGISTRY_NAME+"-layer-id", shadow_layer_id)
    image.set_registry(REGISTRY_NAME+"-radius", chosen_radius)
    image.set_registry(REGISTRY_NAME+"-angle", chosen_angle)
    image.do_end()
    exit()

def button_cancel_click():    
    if image.do_end():
        image.undo()
    layer.select_id(source_layer_id)
    exit()

scale_radius_update_job = None

def scale_radius_update_do():
    global scale_radius_update_job, chosen_radius, scale_radius
    new_radius = scale_radius.get() 
    if new_radius != chosen_radius:
        chosen_radius = new_radius
        apply_blur()
    scale_radius_update_job = None    

def scale_radius_update(event):
    global window, scale_radius_update_job
    if scale_radius_update_job:
        window.after_cancel(scale_radius_update_job)
    scale_radius_update_job = window.after(800, scale_radius_update_do)

scale_angle_update_job = None

def scale_angle_update_do():
    global scale_angle_update_job, chosen_angle, scale_angle
    new_angle = scale_angle.get() 
    if new_angle != chosen_angle:
        chosen_angle = new_angle
        apply_blur()
    scale_angle_update_job = None    

def scale_angle_update(event):
    global window, scale_angle_update_job
    if scale_angle_update_job:
        window.after_cancel(scale_angle_update_job)
    scale_angle_update_job = window.after(800, scale_angle_update_do)

scale_opacity_update_job = None

def scale_opacity_update_do():
    global chosen_opacity 
    new_opacity = scale_opacity.get()
    if new_opacity != chosen_opacity:
        chosen_opacity = new_opacity
        apply_opacity()
    scale_opacity_update_job = None

def scale_opacity_update(event):   
    global window, scale_opacity_update_job
    if scale_opacity_update_job:
        window.after_cancel(scale_opacity_update_job)
    scale_opacity_update_job = window.after(100, scale_opacity_update_do)

window = Tk()
window.title(FRIENDLY_NAME)
window.resizable(False, False)

frame = Frame(window)
frame.pack()

label_radius = Label(frame, text=translation["Radius"])
label_radius.grid(column=0, row=0)
scale_radius = Scale(frame, from_=0, to=MAX_RADIUS, orient=HORIZONTAL, command=scale_radius_update)
scale_radius.grid(column=1, row=0, sticky=W+E, padx=10)
scale_radius.set(chosen_radius)

label_angle = Label(frame, text=translation["Angle"])
label_angle.grid(column=0, row=1)
scale_angle = Scale(frame, from_=0, to=MAX_ANGLE, orient=HORIZONTAL, command=scale_angle_update)
scale_angle.grid(column=1, row=1, sticky=W+E, padx=10)
scale_angle.set(chosen_angle)

label_opacity = Label(frame, text=translation["Opacity"])
label_opacity.grid(column=0, row=2)
scale_opacity = Scale(frame, from_=0, to=MAX_OPACITY, orient=HORIZONTAL, command=scale_opacity_update)
scale_opacity.grid(column=1, row=2, sticky=W+E, padx=10)
scale_opacity.set(chosen_opacity)

frame.columnconfigure(0, pad=20)
frame.columnconfigure(1, minsize=250)
frame.rowconfigure(0, pad=20)
frame.rowconfigure(1, pad=20)
frame.rowconfigure(2, pad=20)

button_ok = Button(window, text=translation["Ok"], command=button_ok_click)
button_ok.pack(side=RIGHT, padx=10, pady=10)
button_cancel = Button(window, text=translation["Cancel"], command=button_cancel_click)
button_cancel.pack(side=RIGHT, pady=10)

image.do_begin()
selection.deselect()
create_shadow_layer()
apply_blur()

window.update()
window_width = window.winfo_width()
screen_width = window.winfo_screenwidth()
window.geometry('+%d+0' % (int((screen_width - window_width) / 2)))

window.lift()
window.attributes('-topmost',True)
window.after_idle(window.attributes,'-topmost',False)

window.mainloop()
button_cancel_click()



# Channels > Make monochromatic
# (fr) Canaux > Rendre monochrome
from lazpaint import image, dialog, layer, filters, colors

translation = dialog.translate_dict(["This is not a chromatic channel"])
  
channel = layer.get_registry("split-channel")
if channel == "R" or channel == "C":
  source = "red"
elif channel == "G" or channel == "M":
  source = "green"
elif channel == "B" or channel == "Y":
  source = "blue"
elif channel == "H":
  layer.fill(colors.GRAY)
  exit()
elif channel is None:
  colors.grayscale()
  exit()
else:
  dialog.show_message(translation["This is not a chromatic channel"])
  exit()

filters.filter_function(red = source, green = source, blue = source)



# Render > Fractal tree
# (fr) Rendu > Arbre fractal
from lazpaint import tools, image, layer, dialog
import math, random

translation = dialog.translate_dict(["Invalid angle", "Vertical size"])

line_buf = []

def line(x, y, x2, y2):
  global line_buf
  if len(line_buf) > 0 and line_buf[-1] == (x, y):
    line_buf.append( (x2, y2) )
  else:
    flush_line()
    line_buf = [(x, y), (x2, y2)]

def flush_line():
  global line_buf
  if len(line_buf) > 0:
    tools.choose(tools.PEN)
    tools.mouse(line_buf)
  line_buf = []

DEG_TO_RAD = math.pi / 180
ANGLE = abs(dialog.input_value(dialog.translate_text("Angle") + " (< 90)", 45))
if ANGLE >= 90:
  dialog.show_message(translation["Invalid angle"])
  exit()
DEFAULT_SIZE_Y = 7*8/2*2 * (1.14+math.cos(ANGLE * DEG_TO_RAD))/2.14
MULTIPLIER = image.get_height() / DEFAULT_SIZE_Y

ZOOM = dialog.input_value(translation["Vertical size"] + " (%)", 95)
MULTIPLIER = MULTIPLIER * ZOOM/100


def drawTree(x1, y1, angle, depth):
    if (depth > 0):
        x2 = x1 + (math.cos(angle * DEG_TO_RAD) * depth * MULTIPLIER)
        y2 = y1 + (math.sin(angle * DEG_TO_RAD) * depth * MULTIPLIER)
        line(x1, y1, x2, y2)
        drawTree(x2, y2, angle - ANGLE, depth - 2)
        drawTree(x2, y2, angle + ANGLE, depth - 2)

image.do_begin()
layer.new()
drawTree(image.get_width() / 2, image.get_height(), -90, 14)
flush_line()
image.do_end()



# Mask > New mask
# (fr) Masque > Nouveau masque
from lazpaint import image, layer, tools, colors, selection, dialog

translation = dialog.translate_dict(["Mask"])

image.do_begin()

selection.deselect()
layer_index = image.get_layer_index()
layer.new(translation["Mask"])
mask_index = image.get_layer_index()
image.move_layer_index(mask_index, layer_index + 1)
layer.set_blend_op(layer.BLEND_MASK)
tools.set_fore_color(colors.WHITE)
tools.set_back_color(colors.BLACK)
tools.choose(tools.FLOOD_FILL)
tools.mouse((0,0), [tools.STATE_LEFT])

image.do_end()



from lazpaint import tools, image, layer, colors

image.new(800, 600)

tools.choose(tools.PEN)
tools.set_pen_width(10)
tools.set_fore_color(colors.RED)
tools.set_back_color(colors.ORANGE)
tools.mouse( (50,50) )
tools.mouse( [(50,100, 0.5), (100,100, 0.5)], [tools.STATE_RIGHT] )

tools.choose(tools.ELLIPSE)
tools.mouse( [(150,50), (250,150)], [tools.STATE_RIGHT] )
tools.set_fore_color(colors.YELLOW)
tools.set_back_color(colors.BLUE)
tools.keys(tools.KEY_RETURN)

tools.choose(tools.PHONG_SHAPE)
tools.set_light_position(500, 500)
tools.mouse( [(50,300), (300,450)] )
tools.set_light_position(50, 600)
tools.keys(tools.KEY_RETURN)

tools.choose(tools.TEXT)
tools.mouse( [(50,150), (450,350)] )
tools.set_fore_color(colors.BLACK)
tools.set_font_style([tools.FONT_STYLE_ITALIC])
tools.set_font_size(20)
tools.set_text_phong(False)
tools.write("Hello\nworld")

#layer.new()
#tools.choose(tools.TEXTURE_MAPPING)
#tools.mouse( [(300,50), (450,200)], [tools.STATE_LEFT, tools.STATE_SHIFT] )
#tools.mouse( [(300,50), (350,80)] )
#tools.keys(tools.KEY_RETURN)




from lazpaint import colors, image

#image.flatten()
#colors.curves(red=[(0, 0), (0.25, 1), (1, 1)])
#colors.curves(lightness=[(0, 0), (0.25, 1), (1, 1)])
#colors.posterize(levels=8, by_lightness=True)
#colors.colorize(hue_angle=180, saturation=0.8, correction=False)
colors.lightness(-1, 0)
#colors.complementary()
#colors.shift(hue_angle=90, saturation=0, correction=True)




from lazpaint import command, dialog

if __name__ == "__main__":
  dialog.show_message("Library to access selection.")

def load(file_name=None):
  command.send("FileLoadSelection", FileName=file_name)

def save_as(file_name=None) -> str:
  return command.send("FileSaveSelectionAs?", FileName=file_name)

def invert():
  command.send("EditInvertSelection")

def deselect():
  command.send("EditDeselect")

def copy():
  command.send("EditCopy")

def cut():
  command.send("EditCut")

def delete():
  command.send("EditDeleteSelection")

def select_all():
  command.send("EditSelectAll")

def fit():
  command.send("EditSelectionFit")

def is_mask_empty() -> bool:
  return command.send("IsSelectionMaskEmpty?")

def is_layer_empty() -> bool:
  return command.send("IsSelectionLayerEmpty?")

def paste():
  command.send("EditPaste")





from lazpaint import tools, image, colors, dialog

tools.choose(tools.GRADIENT)
tools.mouse([
    (0,0), (image.get_width()/2, image.get_height()/2)
  ])
tools.set_back_gradient_colors([colors.GREEN, colors.RED])
tools.set_back_gradient_interpolation(tools.GRADIENT_INTERPOLATION_LINEAR_HSL_POSITIVE)
tools.set_back_gradient_repetition(tools.GRADIENT_REPETITION_REPEAT)
tools.keys([tools.KEY_RETURN])

dialog.show_message(tools.get_back_gradient_colors())





# Merge RGB channels
from lazpaint import image, dialog, layer

try:
    from PIL import Image
except ImportError:
    dialog.show_message("Please install Pillow for Python.")
    exit()

channels_id = None
image.do_begin()

if layer.get_registry("split-channel") is not None:
  layer_id = layer.get_registry("split-source-id")
  if image.contains_layer_id(layer_id):
    layer.select_id(layer_id)
    channels_id = layer.get_registry("split-channels-id")
  else:
    channels_id = []
    for i in range(1, image.get_layer_count()+1):
      image.select_layer_index(i)
      cur_layer_id = layer.get_registry("split-source-id")
      if cur_layer_id == layer_id:
        channels_id.append(layer.get_id())      
else:
  layer_id = layer.get_id()
  channels_id = layer.get_registry("split-channels-id")

if channels_id is None:
  dialog.show_message("Current layer is not split")
  exit()

width, height = image.get_size()
zero = Image.new("L", (width, height))
one = Image.new("L", (width, height), 255)
channels = {"R": zero, "G": zero, "B": zero, "A": one}

temp_name = image.get_temporary_name()
for ch_id in channels_id:
  if image.contains_layer_id(ch_id):
    layer.select_id(ch_id)
    ch = layer.get_registry("split-channel")
    if ch == "A":
      file_format = "PNG"
    else:
      file_format = "BMP"
    temp_name = layer.save_as(temp_name, file_format)
    if temp_name is None:
      dialog.show_message("Failed to retrieve layer")
      exit()
    channels[ch] = Image.open(temp_name).getchannel(ch)

merged = Image.merge("RGBA", [channels["R"], channels["G"], channels["B"], channels["A"]])
merged.save(temp_name, "PNG")

for ch_id in channels_id:
  if image.contains_layer_id(ch_id):
    layer.select_id(ch_id)
    layer.remove()
 
layer.add_from_file(temp_name)
layer.set_name("Merged channels")
new_layer_id = layer.get_id()
new_layer_index = image.get_layer_index()

if image.contains_layer_id(layer_id):
  layer.select_id(layer_id)
  layer_index = image.get_layer_index()
  layer_name = layer.get_name()
  layer_opacity = layer.get_opacity()
  image.move_layer_index(new_layer_index, layer_index+1)
  layer.select_id(layer_id)
  layer.remove()
  layer.select_id(new_layer_id)
  layer.set_name(layer_name)
  layer.set_opacity(layer_opacity)
  
image.do_end()



from lazpaint import image, layer, dialog

def show_frame():
  dialog.show_message("Frame " + str(image.get_frame_index()) + '/' + str(image.get_frame_count()) )

show_frame()

image.new(128,128, 0)
image.save_as("test_multi.tiff")
show_frame()

image.new_frame()
layer.fill(64)
image.save()
show_frame()

image.new_frame()
layer.fill(128)
image.save()
show_frame()

image.new_frame()
layer.fill(192)
image.save()
show_frame()

image.new_frame()
layer.fill(255)
image.save()
show_frame()

image.load_frame(1)
show_frame()

